<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\Session\Session;
use Teknisa\MAN\Utilities;
use Zeedhi\Framework\Util\Functions;

class FechamentoCaixaController extends CrudWithParameterBag {
    
    protected $dataSourceName = 'vnd00600_fechamentocaixa';
    protected $parameterBagColumns = array('CDFILIAL', 'DTINICIAL', 'DTFINAL', 'NRORG','ABERTO','FECHADO');
    protected $environment;
    protected $FechamentoCaixaService;
    
    public function __construct(Manager $dataSourceManager, ParameterBag $parameterBag,$environment, $FechamentoCaixaService,Utilities\Util $utilService) {
        parent::__construct($dataSourceManager, $parameterBag);
        $this->environment = $environment;
        $this->utilService = $utilService;
        $this->FechamentoCaixaService = $FechamentoCaixaService;
    }
    
    public function findFilial(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'vnd00600_fechamentocaixa_filial' ;
        $this->find($request,$response);
    }
    
    public function findLoja(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'vnd00600_fechamentocaixa_loja' ;
        $this->find($request,$response);
    }
    public function findCaixa(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'vnd00600_fechamentocaixa_caixa' ;
        $this->find($request,$response);
    }
    
    public function fecharCaixa(DTO\Request $request, DTO\Response $response){
        $row = $request->getDataSet()->getRows();
        $currentTime = new \DateTime(null,new \DateTimeZone('America/Sao_Paulo'));
        $currentTime = $currentTime->format("d/m/Y H:i:s");
        $CDFILIAL       =   $row[0]['CDFILIAL'];
        $CDCAIXA        =   $row[0]['CDCAIXA'];
        $DTABERCAIX     =   $row[0]['DTABERCAIX'];
        $DTFECHCAIX     =   $row[0]['DTFECHCAIX'];
        $CDOPERFECH     =   $this->environment->getCdOperador();
        $CDOPERADOR     =   $row[0]['CDOPERADOR'];
        $NRORG          =   $this->environment->getNrOrgTrab();
        $DTULTATU       =   $currentTime;
        $NRORGULTATU    =   $this->environment->getNrOrgTrab();
        $CDOPERULTATU   =   $this->environment->getCdOperador();
        $DSOBSCAIXA     =   (isset($row[0]['DSOBSCAIXA'])) ? $row[0]['DSOBSCAIXA'] : null;
        
        $vendaPendenteExists = $this->FechamentoCaixaService->vendaPendenteExists($CDFILIAL,$CDCAIXA,$DTABERCAIX,$NRORG);
        
        if ($vendaPendenteExists){
            $response->addMessage(new Message("Operação não permitida. Este Caixa possui venda pendente.", Message::TYPE_ERROR));
        } else {
            try{
                
                $this->FechamentoCaixaService->fecharCaixa($CDFILIAL,$CDCAIXA,$DTABERCAIX,$DTFECHCAIX,$CDOPERFECH,$CDOPERADOR,$NRORG,$DTULTATU,$NRORGULTATU,$CDOPERULTATU,$DSOBSCAIXA);
                $return = [
                    'CDOPERFECH' => $CDOPERFECH,
                    'DTFECHCAIX' => $DTFECHCAIX
                ];
                
                $response->addDataSet(new DataSet($this->dataSourceName, $return));
                $response->addMessage(new Message("Caixa fechado com sucesso!"));
            } catch(Exception $e){
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
        
        
    }
    
    public function deleteFechamento(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $currentTime = new \DateTime(null,new \DateTimeZone('America/Sao_Paulo'));
        $currentTime = $currentTime->format("d/m/Y H:i:s");
        $CDFILIAL        =   $row->get('CDFILIAL');
        $CDCAIXA         =   $row->get('CDCAIXA');
        $DTABERCAIX      =   $row->get('DTABERCAIX');
        $DTFECHCAIXCP    =   Functions::arrayKeyExists("DTFECHCAIXCP", $row) ? $row['DTFECHCAIXCP'] : NULL;
        $CDOPERADOR      =   $row->get('CDOPERADOR');
        $NRORG           =   $this->environment->getNrOrgTrab();
        $DTULTATU        =   $currentTime;
        $NRORGULTATU     =   $this->environment->getNrOrgTrab();
        $CDOPERULTATU    =   $this->environment->getCdOperador();
        $IDTPEMISSAOFOS  =   $row->get('IDTPEMISSAOFOS');

        
        
        $isReducaoZApproved = $this->FechamentoCaixaService->isReducaoZApproved($CDFILIAL,$CDCAIXA,$DTABERCAIX,$NRORG);
        $isNFCeApproved = $this->FechamentoCaixaService->isNFCeApproved($CDFILIAL,$CDCAIXA,$DTABERCAIX,$NRORG);
        
        
        //Se o caixa não for ECF (IDTPEMISSAOFOS), haverá validação para saber se NFCe foi aprovado
        if ( $isReducaoZApproved || ($isNFCeApproved && $IDTPEMISSAOFOS != 'ECF') ){
            if($isReducaoZApproved)
                $response->addMessage(new Message('Operação não permitida. Redução Z do caixa selecionado já foi aprovada.', Message::TYPE_ERROR));
            else if($isNFCeApproved)
                $response->addMessage(new Message('Operação não permitida. As vendas NFC-e ou CF-e-SAT já foram aprovadas.', Message::TYPE_ERROR));
        } else {
            try{
                $this->FechamentoCaixaService->deleteFechamento($CDFILIAL,$CDCAIXA,$DTABERCAIX,$CDOPERADOR,$NRORG,$DTULTATU,$NRORGULTATU,$CDOPERULTATU,$DTFECHCAIXCP);
                $return = [
                    'CDCAIXA' => $CDCAIXA
                ];
                $response->addMessage(new Message('Operação realizada com sucesso!'));
                $response->addDataSet(new DataSet($this->dataSourceName, $return));
            } catch(Exception $e){
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }
    
    public function findFechementoCaixa(DTO\Request\Filter $request, DTO\Response $response) {
        
        $conditions = $this->utilService->getParamsAsArray($request);
        if( array_key_exists('DTABERTURA', $conditions) ) {
        
            $conditions['DTINICIAL'] = $conditions['DTABERTURA'][0];
            $conditions['DTFINAL'] = $conditions['DTABERTURA'][1];
            
            unset($conditions['DTABERTURA']);
        }
        

        $dataSourceFilter = array();
        
        foreach($conditions as $key => $value) {
            $dataSourceFilter[] = $condition = array(
                "columnName" => $key,
                "operator"=> "=",
                "value"=> $value
            );
        }
        
        $original = $request->getFilterCriteria()->getConditions();
            
        for($i = 0 ; $i < count($dataSourceFilter) ; $i++) {
            for($j = 0 ; $j < count($original) ; $j++ ) {
                if( $dataSourceFilter[$i]['columnName'] === $original[$j]['columnName'] &&
                    $dataSourceFilter[$i]['operator'] !== $original[$j]['operator'])  {
                    $dataSourceFilter[$i]['operator'] = $original[$j]['operator'];
                }
                
            }
        }
        
        $newRequest = new DTO\Request\Filter(
            new FilterCriteria(
                $request->getFilterCriteria()->getDataSourceName(),
                $dataSourceFilter,
                $request->getFilterCriteria()->getPage(),
                $request->getFilterCriteria()->getPageSize()
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
        parent::find($newRequest, $response);
    }
}